/*
 * @(#) $Id:  $
 *
 * Copyright 2009/2010 by sIT Solutions,
 * A-1110 Wien, Geiselbergstr.21-25.
 * All rights reserved.
 *
 */
package org.openxma.dsl.core.linking;

import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.xtext.TerminalRule;
import org.eclipse.xtext.diagnostics.IDiagnosticConsumer;
import org.eclipse.xtext.linking.lazy.LazyLinker;
import org.eclipse.xtext.parsetree.AbstractNode;
import org.eclipse.xtext.parsetree.CompositeNode;
import org.eclipse.xtext.parsetree.LeafNode;
import org.eclipse.xtext.parsetree.NodeUtil;
import org.openxma.dsl.core.linking.support.JavaDocStringBuilder;
import org.openxma.dsl.core.linking.support.SiblingUtil;
import org.openxma.dsl.core.model.IDocumentable;

/**
 * This linker parses JavaDoc comments between elements and adds them to related model elements
 * @author s6526  
 * @since version_number
 */
public class DocumentationLinker extends LazyLinker {
    
    @Override
    protected void afterModelLinked(EObject model, IDiagnosticConsumer diagnosticsConsumer) {
        try {
            findJavaDocComments(NodeUtil.getNode(model), diagnosticsConsumer);
        }catch (Throwable t) {
            System.out.println("Exception in DocumentationLinker " + t.toString());
            t.printStackTrace(System.out);
        }
    }
    
    void findJavaDocComments(CompositeNode rootNode, IDiagnosticConsumer diagnosticsConsumer) {
        TreeIterator<EObject> iterator = rootNode.eAllContents();
        JavaDocStringBuilder javaDocStringBuilder = new JavaDocStringBuilder();
        
        while (iterator.hasNext()) {
            EObject abstractNode = iterator.next();
            if (abstractNode instanceof LeafNode && ((LeafNode)abstractNode).getGrammarElement() instanceof TerminalRule) {
                javaDocStringBuilder.newText(((LeafNode) abstractNode).getText());
                if (javaDocStringBuilder.isJavaDocCommentCompleted()) {
                    String javaDocString = javaDocStringBuilder.getJavaDocAndReset();
                    attachJavaDocOnNextDocumentableElement(javaDocString, ((LeafNode)abstractNode));
                }
            }
            else {
                javaDocStringBuilder.reset();
            }
        }
    }

    private void attachJavaDocOnNextDocumentableElement(String javaDocString, AbstractNode abstractNode) {
        AbstractNode nextSibling = SiblingUtil.getNextSibling(abstractNode);
        IDocumentable documentableElement = findIDocumentableInHierarchy(NodeUtil.findASTElement(nextSibling));
        
        if (documentableElement != null && documentableElement.getTxtDescription() == null) {  
            documentableElement.setTxtDescription(javaDocString);
        }
    }
    /**
     *  
     */
    private IDocumentable findIDocumentableInHierarchy(EObject semanticElement) {
        if (semanticElement == null)
            return null;
        if (semanticElement instanceof IDocumentable)
            return (IDocumentable) semanticElement;
        return findIDocumentableInHierarchy(semanticElement.eContainer());
    }


}
